﻿//-------------------------------------------------------------------------------------
// Example of custom classes for accounts and folders management.
// About how to attach these classes to application see above (Session_Start)
//-------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Configuration;
using System.Reflection;
using System.Security.Principal;
using HttpCommander;
using Newtonsoft.Json;
using RE = System.Text.RegularExpressions;

/// <summary>
/// Simple account manager class.
/// </summary>
public sealed class SimpleAccountManager : IAccountManager
{
    #region Fields

    private string __ldapContainer;
    private ReadMembership __readMembership = ReadMembership.None;
    private List<User> __users = null;
    private Dictionary<SecurityIdentifier, User> __sidsUsers = null;
    private List<Group> __groups = null;
    private Dictionary<SecurityIdentifier, Group> __sidsGroups = null;
    private bool __isADVersion = true;

    #endregion

    #region Constructors

    /// <summary>
    /// Initializes a new instance <see cref="HttpCommander.SimpleAccountManager"/> class
    /// with the specified list users and list groups.
    /// </summary>
    /// <param name="users">List users.</param>
    /// <param name="groups">List groups.</param>
    public SimpleAccountManager(List<User> users, List<Group> groups)
    {
        this.__isADVersion = Utils.IsADVersion;

        if (this.__isADVersion)
        {
            this.__ldapContainer = ConfigurationManager.AppSettings["ldapContainer"];
            string s = ConfigurationManager.AppSettings["readWindowsUsersGroupMembership"];
            if (!string.IsNullOrEmpty(s))
            {
                if (string.Compare(s, "no", true) == 0 || string.Compare(s, "none", true) == 0) this.__readMembership = ReadMembership.None;
                else if (string.Compare(s, "service", true) == 0) this.__readMembership = ReadMembership.Service;
                else if (string.Compare(s, "com", true) == 0) this.__readMembership = ReadMembership.Com;
                else if (string.Compare(s, "current", true) == 0) this.__readMembership = ReadMembership.Current;
                else this.__readMembership = ReadMembership.None;
            }
        }
        else
        {
            if (users != null)
                this.__users = users;
            if (groups != null)
                this.__groups = groups;
        }        
    }

    /// <summary>
    /// Initializes a new instance <see cref="HttpCommander.SimpleAccountManager"/> class
    /// with the specified list users and groups from windows groups.
    /// </summary>
    /// <param name="users">List users.</param>
    public SimpleAccountManager(List<User> users) : this(users, null) { }

    /// <summary>
    /// Initializes a new instance <see cref="HttpCommander.SimpleAccountManager"/> class
    /// with users and groups from windows.
    /// </summary>
    public SimpleAccountManager() : this (null, null) { }

    #endregion 

    #region IAccountManager Members

    /// <summary>
    /// Get list with groups names for specified user.
    /// </summary>
    /// <param name="userName">User name.</param>
    /// <returns>List groups names.</returns>
    public List<string> GetGroupsNames(string userName)
    {
        List<string> _list = new List<string>();
        
        if (string.IsNullOrEmpty(userName))
            return _list;

        if (this.__isADVersion)
            try
            {
                switch (this.__readMembership)
                {
                    case ReadMembership.None:
                        break;
                    case ReadMembership.Com:
                        Type mainclass = Type.GetTypeFromProgID("HTTPCommanderGroupsReader.MainClass");
                        object comWorker = Activator.CreateInstance(mainclass);
                        object[] myargs = { 0, 0, 0 };
                        object RetVal = null;
                        if (string.Compare(this.__ldapContainer, "", true) == 0)
                        {
                            myargs[0] = 6;
                            myargs[1] = Utils.CurrentUserDomain;
                            myargs[2] = userName; //Utils.CurrentUserName;
                            RetVal = mainclass.InvokeMember("ADGetInfo", BindingFlags.InvokeMethod, null, comWorker, myargs);
                        }
                        else
                        {
                            myargs[0] = 6;
                            myargs[1] = this.__ldapContainer;
                            myargs[2] = userName; //Utils.CurrentUserName;
                            RetVal = mainclass.InvokeMember("LDAPADGetInfo", BindingFlags.InvokeMethod, null, comWorker, myargs);
                        }
                        System.Object[] vl = (System.Object[])RetVal;
                        for (int i = 0; i < vl.Length; i++)
                        {
                            object val = vl[i];
                            if (val != null)
                                _list.Add(val.ToString());
                        }
                        break;
                    case ReadMembership.Current:
                        _list = new List<string>(ADAccountManager.GetGroups(userName, Utils.CurrentUserDomain, this.__ldapContainer, Utils.UseUniversalWayToReadGroups));
                        break;
                }
            }
            catch (Exception ex)
            {
                Utils.LogError(ex);
            }
        else if (this.__groups != null)
        {
            List<Group> _glist = this.__groups.FindAll(delegate(Group g)
            {
                return g.Users.Contains(userName);
            });
            foreach (Group g in _glist)
                _list.Add(g.Name);
        }

        return _list;
    }

    public void SetPassword(string userName, string password)
    {
        throw new NotImplementedException();
    }

    /// <summary>
    /// Get list users.
    /// </summary>
    /// <returns>List users.</returns>
    public List<User> GetUsers()
    {
        if (this.__isADVersion)
        {
            if (this.__sidsUsers != null)
            {
                List<User> result = new List<User>();
                foreach (User user in this.__sidsUsers.Values)
                    result.Add(user);
                return result;
            }
            
            try
            {
                Dictionary<SecurityIdentifier, string> sidsusers = null;
                switch (this.__readMembership)
                {
                    case ReadMembership.None:
                        break;
                    case ReadMembership.Com:
                        break;
                    case ReadMembership.Current:
                        sidsusers = ADAccountManager.GetUsers(Utils.CurrentUserDomain, this.__ldapContainer);
                        break;
                    default:
                        break;
                }
                this.__sidsUsers = new Dictionary<SecurityIdentifier, User>();
                List<User> adUsers = new List<User>();
                if (sidsusers != null)
                    foreach (SecurityIdentifier sid in sidsusers.Keys)
                    {
                        User u = new User(sidsusers[sid], null);
                        this.__sidsUsers[sid] = u;
                        adUsers.Add(u);
                    }
                return adUsers;
            }
            catch (Exception ex)
            {
                Utils.LogError(ex);
            }
        }
        else
        {
            if (this.__users != null)
                return this.__users;
            this.__users = new List<User>();
            return this.__users;
        }

        return new List<User>();
    }

    /// <summary>
    /// Get list groups.
    /// </summary>
    /// <returns>List groups.</returns>
    public List<Group> GetGroups()
    {
        if (this.__isADVersion)
        {
            if (this.__sidsGroups != null)
            {
                List<Group> result = new List<Group>();
                foreach (Group group in this.__sidsGroups.Values)
                    result.Add(group);
                return result;
            }

            try
            {
                Dictionary<SecurityIdentifier, string> sidsgroups = null;
                switch (this.__readMembership)
                {
                    case ReadMembership.None:
                        break;
                    case ReadMembership.Com:
                        break;
                    case ReadMembership.Current:
                        sidsgroups = ADAccountManager.GetWholeDomainGroups(Utils.CurrentUserDomain, this.__ldapContainer);
                        break;
                    default:
                        break;
                }
                this.__sidsGroups = new Dictionary<SecurityIdentifier, Group>();
                List<Group> adGroups = new List<Group>();
                if (sidsgroups != null)
                    foreach (SecurityIdentifier sid in sidsgroups.Keys)
                    {
                        Group g = new Group(sidsgroups[sid]);
                        this.__sidsGroups[sid] = g;
                        adGroups.Add(g);
                    }
                return adGroups;
            }
            catch (Exception ex)
            {
                Utils.LogError(ex);
            }
        }
        else
        {
            if (this.__groups != null)
                return this.__groups;
            this.__groups = new List<Group>();
            return this.__groups;
        }

        return new List<Group>();
    }

    /// <summary>
    /// Get home folder for specified user name.
    /// </summary>
    /// <param name="userName">User name.</param>
    /// <returns>Home folder.</returns>
    /// <exception cref="System.NotImplementedException">
    /// This method not implemented.
    /// Always generate exception "NotImplementedException".
    /// </exception>
    public string GetHomeFolder(string userName)
    {
        if (string.IsNullOrEmpty(userName) || !this.__isADVersion)
            return null;
        
        try
        {
            switch (this.__readMembership)
            {
                case ReadMembership.None:
                    return null;
                case ReadMembership.Com:
                    Type mainclass = Type.GetTypeFromProgID("HTTPCommanderGroupsReader.MainClass");
                    object comWorker = Activator.CreateInstance(mainclass);
                    object[] myargs = { 0, 0, 0 };
                    object RetVal = null;
                    if (string.Compare(this.__ldapContainer, "", true) == 0)
                    {
                        myargs[0] = 3;
                        myargs[1] = Utils.CurrentUserDomain;
                        myargs[2] = userName; //Utils.CurrentUserName;
                        RetVal = mainclass.InvokeMember("ADGetInfo", BindingFlags.InvokeMethod, null, comWorker, myargs);
                    }
                    else
                    {
                        myargs[0] = 3;
                        myargs[1] = this.__ldapContainer;
                        myargs[2] = userName; //Utils.CurrentUserName;
                        RetVal = mainclass.InvokeMember("LDAPADGetInfo", BindingFlags.InvokeMethod, null, comWorker, myargs);
                    }
                    return (string)RetVal;
                case ReadMembership.Current:
                    return ADAccountManager.GetHomeFolder(userName, Utils.CurrentUserDomain, this.__ldapContainer);
                default:
                    return null;
            }
        }
        catch (Exception ex)
        {
            Utils.LogError(ex);
            return null;
        }
    }

    /// <summary>
    /// Check user with specified name and password.
    /// </summary>
    /// <param name="name">User name.</param>
    /// <param name="password">User password.</param>
    /// <returns>True, if exists user with specified user name and password, else False.</returns>
    public ValidateUserResult ValidateUser(string name, string password)
    {
        if (LoginAttempts.Instance.IsUserBlocked(name))
            return ValidateUserResult.Blocked;

        if (this.__isADVersion)
            try
            {
                switch (this.__readMembership)
                {
                    case ReadMembership.None:
                        return ValidateUserResult.Failure;
                    case ReadMembership.Com:
                        return ValidateUserResult.Failure;
                    case ReadMembership.Current:
                        return ADAccountManager.ValidateUser(name, password) ? ValidateUserResult.Success : ValidateUserResult.Failure;
                    default:
                        return ValidateUserResult.Failure;
                }
            }
            catch (Exception ex)
            {
                Utils.LogError(ex);
                return ValidateUserResult.Failure;
            }
        else if (this.__users == null)
            this.__users = new List<User>();

        User user = GetUser(name);

        bool res = user != null && user.TestPassword(password);
        if (res) LoginAttempts.Instance.SuccessfulAttempt(name);
        else LoginAttempts.Instance.UnsuccessfulAttempt(name);
        return res ? ValidateUserResult.Success : ValidateUserResult.Failure;        
    }

    public bool ExistsUser(string name)
    {
        if (string.IsNullOrEmpty(name))
            return false;

        List<User> searchList = this.GetUsers();

        return searchList.Exists(delegate(User u)
        {
            return User.EqualUserNames(u.Name, name);
        });
    }

    public bool UserHasEmail(string name)
    {
        if (string.IsNullOrEmpty(name)) return false;
        User user = GetUser(name);
        if (user == null) return false;
        return !string.IsNullOrEmpty(user.Email);
    }

    public bool EmailUserPassword(string name)
    {
        throw new NotImplementedException();
    }

    public void AddUserWithDefaultGroups(User user)
    {
        throw new NotImplementedException();
    }

    /// <summary>
    /// Check user with specified name and password hash.
    /// </summary>
    /// <param name="name">User name.</param>
    /// <param name="passwordHash">User password hash.</param>
    /// <returns>True, if exists user with specified user name and password hash, else False.</returns>
    public ValidateUserResult ValidateUserHash(string name, string passwordHash)
    {
        if (LoginAttempts.Instance.IsUserBlocked(name))
            return ValidateUserResult.Blocked;
        if (this.__isADVersion)
            return ValidateUserResult.Failure;
        else if (this.__users == null)
            this.__users = new List<User>();

        User user = GetUser(name);
        bool res = user != null && user.TestPasswordHash(passwordHash);
        if (res) LoginAttempts.Instance.SuccessfulAttempt(name);
        else LoginAttempts.Instance.UnsuccessfulAttempt(name);
        return res ? ValidateUserResult.Success : ValidateUserResult.Failure;
    }

    /// <summary>
    /// Add new user.
    /// </summary>
    /// <param name="user"><see cref="HttpCommander.User"/> object.</param>
    /// <exception cref="System.Exception">
    /// If specified user already exists, then generate
    /// exception with message "The user already exists".
    /// </exception>
    public void AddUser(User user)
    {
        if (ExistsUser(user.Name))
            throw new Exception("The user '" + user.Name + "' already exists.");

        if (this.__isADVersion)
            throw new NotImplementedException();
        else if (this.__users == null)
            this.__users = new List<User>();

        this.__users.Add(user);
    }

    /// <summary>
    /// Delete the specified user.
    /// </summary>
    /// <param name="userName">User name.</param>
    public void DeleteUser(string userName)
    {
        if (string.IsNullOrEmpty(userName))
            return;

        if (this.__isADVersion)
            throw new NotImplementedException();
        else if (this.__users == null)
            this.__users = new List<User>();

        User _user = GetUser(userName);
        if (_user != null)
        {
            this.__users.Remove(_user);
            if (this.__groups != null)
                foreach (Group group in this.__groups)
                    if (group.Users.Contains(userName))
                        group.Users.Remove(userName);
        }
    }

    /// <summary>
    /// Update specified user.
    /// </summary>
    /// <param name="user"><see cref="HttpCommander.User"/> object.</param>
    public void UpdateUser(User user, bool preservePassword)
    {
        if (this.__isADVersion)
            throw new NotImplementedException();
        else if (this.__users == null)
            this.__users = new List<User>();

        int index = this.__users.IndexOf(user);
        if (index < 0) this.AddUser(user);
        else
        {
            if (preservePassword)
            {
                user.Password = this.__users[index].Password;
                user.PasswordHash = this.__users[index].PasswordHash;
            }
            user.NormalizePassword();
            this.__users[index] = user;
        }
    }

    public void RenameUser(string oldUserName, string newUserName)
    {
        throw new NotImplementedException();
    }

    /// <summary>
    /// Change password for specified user.
    /// </summary>
    /// <param name="userName">User name.</param>
    /// <param name="oldPassword">Old password.</param>
    /// <param name="newPassword">New password.</param>
    /// <exception cref="System.ArgumentNullException">
    /// If <b>userName</b> is null or empty.
    /// </exception>
    /// <exception cref="System.ArgumentException">
    /// If <b>userName</b> not found.
    /// </exception>
    /// <exception cref="System.Exception">
    /// If <b>oldPassword</b> is invalid.
    /// </exception>
    public void ChangePassword(string userName, string oldPassword, string newPassword)
    {
        if (string.IsNullOrEmpty(userName))
            throw new ArgumentNullException("userName");

        if (this.__isADVersion)
        {
            if (this.__readMembership != ReadMembership.Current)
                throw new NotImplementedException();
            ADAccountManager.ChangeUserPassword(userName, Utils.CurrentUserDomain, this.__ldapContainer, oldPassword, newPassword);
            return;
        }
        else if (this.__users == null)
            this.__users = new List<User>();
        
        User _user = GetUser(userName);
        if (_user == null)
            throw new ArgumentException("User '" + userName + "' not found.");
        if (!_user.TestPassword(oldPassword))
            throw new Exception("Wrong password!");
        _user.SetPassword(newPassword);
    }

    /// <summary>
    /// Add new group.
    /// </summary>
    /// <param name="group"><see cref="HttpCommander.Group"/> object.</param>
    /// <exception cref="System.Exception">
    /// If <b>group</b> already exists.
    /// </exception>
    public void AddGroup(Group group)
    {
        if (this.__isADVersion)
            throw new NotImplementedException();
        else if (this.__groups == null)
            this.__groups = new List<Group>();

        if (group != null)
        {
            if (this.__groups.Contains(group))
                throw new Exception("The group '" + group.Name + "' already exists.");
            this.__groups.Add(group);
        }
    }

    /// <summary>
    /// Delete the specified group.
    /// </summary>
    /// <param name="groupName">Group name.</param>
    public void DeleteGroup(string groupName)
    {
        if (string.IsNullOrEmpty(groupName))
            return;

        if (this.__isADVersion)
            throw new NotImplementedException();
        else if (this.__groups == null)
            this.__groups = new List<Group>();

        Group _group = this.__groups.Find(delegate(Group g)
        {
            return g.Name.Equals(groupName, StringComparison.InvariantCultureIgnoreCase);
        });
        if (_group != null)
            this.__groups.Remove(_group);
    }

    /// <summary>
    /// Update the specified group.
    /// </summary>
    /// <param name="group"><see cref="HttpCommander.Group"/> object.</param>
    public void UpdateGroup(Group group)
    {
        if (this.__isADVersion)
            throw new NotImplementedException();
        else if (this.__groups == null)
            this.__groups = new List<Group>();

        if (group != null)
        {
            int index = this.__groups.IndexOf(group);
            if (index < 0) this.AddGroup(group);
            else this.__groups[index] = group;
        }
    }

    public void RenameGroup(string oldGroupName, string newGroupName)
    {
        throw new NotImplementedException();
    }

    /// <summary>
    /// Get user by name.
    /// </summary>
    /// <param name="userName">User name.</param>
    /// <returns>User object.</returns>
    public User GetUser(string userName)
    {
        if (string.IsNullOrEmpty(userName))
            return null;

        List<User> searchList = this.GetUsers();

        User _user = searchList.Find(delegate(User u)
        {
            return User.EqualUserNames(u.Name, userName);
        });

        return _user;
    }

    #endregion
}

/// <summary>
/// Simple folder manager class.
/// </summary>
public class SimpleFolderManager : IFolderManager
{
    #region Fields

    private List<Folder> __folders = new List<Folder>();

    #endregion

    #region Constructors

    /// <summary>
    /// Initializes a new instance <see cref="HttpCommander.SimpleFolderManager"/> class
    /// with specified list folders
    /// </summary>
    /// <param name="folders">List folders.</param>
    public SimpleFolderManager(List<Folder> folders)
    {
        if (folders != null)
            this.__folders = folders;
    }

    /// <summary>
    /// Initializes a new instance <see cref="HttpCommander.SimpleFolderManager"/> class
    /// with empty list folders.
    /// </summary>
    public SimpleFolderManager() { }

    #endregion

    #region IFolderManager Members

    /// <summary>
    /// Get list folders.
    /// </summary>
    /// <returns>List folders.</returns>
    public List<Folder> GetFolders()
    {
        return this.__folders;
    }

    /// <summary>
    /// Get folders for specifided user and with
    /// permission specified list group names.
    /// </summary>
    /// <param name="userName">User name.</param>
    /// <param name="groups">List group names.</param>
    /// <returns></returns>
    public List<Folder> GetFolders(string userName, List<string> groups)
    {
        List<Folder> _list = new List<Folder>();
        if (string.IsNullOrEmpty(userName))
            return _list;
        foreach (Folder folder in this.__folders)
        {
            foreach (Permission permission in folder.UserPermissions)
            {
                bool match = permission.IdentityName.ToLower().Contains("regex:") ?
                    RE.Regex.Match(userName, permission.IdentityName.Substring(6), RE.RegexOptions.IgnoreCase).Success :
                    string.Compare(userName, permission.IdentityName, true) == 0;

                if (match && !_list.Contains(folder))
                    _list.Add(folder);
            }
            foreach (Permission permission in folder.GroupPermissions)
                foreach (string group in groups)
                {

                    bool match = permission.IdentityName.ToLower().Contains("regex:") ?
                        RE.Regex.Match(group, permission.IdentityName.Substring(6), RE.RegexOptions.IgnoreCase).Success :
                        string.Compare(group, permission.IdentityName, true) == 0;

                    if (match)
                    {
                        Utils.MatchedGroup = group;
                        if (!_list.Contains(folder))
                            _list.Add(folder);
                    }
                }
        }
        return _list;
    }

    /// <summary>
    /// Delete folder with specified name.
    /// </summary>
    /// <param name="folderName">Folder name.</param>
    public void DeleteFolder(string folderName)
    {
        if (string.IsNullOrEmpty(folderName))
            return;
        Folder folder = this.__folders.Find(delegate(Folder f)
        {
            return f.Name.Equals(folderName, StringComparison.InvariantCultureIgnoreCase);
        });
        if (folder != null)
            this.__folders.Remove(folder);
    }

    /// <summary>
    /// Add new folder.
    /// </summary>
    /// <param name="folder"><see cref="HttpCommander.Folder"/> object.</param>
    /// <exception cref="System.Exception">
    /// If <b>folder</b> already exists.
    /// </exception>
    public void AddFolder(Folder folder)
    {
        if (this.__folders.Exists(delegate(Folder f) { return f.Equals(folder); }))
            throw new Exception("Folder '" + folder.Name + "' already exists.");

        this.__folders.Add(folder);
    }

    /// <summary>
    /// Add new folder from serialized data.
    /// </summary>
    /// <param name="data">Serialized data.</param>
    /// <exception cref="System.Exception">
    /// If deserialised folder already exists.
    /// </exception>
    public void AddFolder(string data)
    {
        Folder folder = JsonConvert.DeserializeObject<Folder>(data);
        this.AddFolder(folder);
    }

    /// <summary>
    /// Update specified folder.
    /// </summary>
    /// <param name="folder"><see cref="HttpCommander.Folder"/> object.</param>
    public void UpdateFolder(Folder folder)
    {
        int index = this.__folders.IndexOf(folder);
        if (index < 0)
            this.AddFolder(folder);
        else
        {
            this.__folders[index] = folder;
        }
    }

    /// <summary>
    /// Update folder from specified serialized data.
    /// </summary>
    /// <param name="data">Serialized data.</param>
    public void UpdateFolder(string data)
    {
        Folder folder = JsonConvert.DeserializeObject<Folder>(data);
        this.UpdateFolder(folder);
    }

    #endregion
}